@@ -291,6 +291,7 @@ char* xstrdup(const char *string) {
 /* asprintf() with out-of-memory checking.  Also fail if formatting fails so
  * that the caller doesn't have to check any error return.
  */
+__attribute__((format (printf, 1, 2)))
 char* xasprintf(const char* fmt, ...) {
     va_list	args;
     int		ret;
@@ -308,7 +309,7 @@ char* xasprintf(const char* fmt, ...) {
 
 /* Create an empty list-entry
  */
-struct _node* create_node() {
+struct _node* create_node(void) {
     struct _node*	newnode;
 
     newnode=(struct _node*)xmalloc(sizeof(struct _node));
@@ -683,7 +684,7 @@ int fputpwent(const struct passwd *passwd, FILE * f) {
 
 
 /* Simple function to print usage information */
-void usage() {
+void usage(void) {
     printf(
 	"Usage: update-passwd [OPTION]...\n"
 	"\n"
@@ -713,7 +714,7 @@ void usage() {
 
 /* Simple function to print our name and version
  */
-void version() {
+void version(void) {
     printf("update-passwd %s\n", PACKAGE_VERSION);
 }
 
@@ -758,6 +759,8 @@ char* escape_debconf(const char* string) {
     char*	copy;
     char*       p;
 
+    assert(string);
+
     copy=xstrdup(string);
     for (p=copy; *p!='\0'; p++)
 	if (!isalnum((int)*p) && *p!='-' && *p!='_')
@@ -1145,11 +1148,14 @@ int write_passwd(const struct _node* passwd, const char* file) {
     if (opt_verbose>2)
 	printf("Writing passwd-file to %s\n", file);
 
+    /* Delete possible left-over file */
+    (void) unlink(file);
+
     if (selinux_prepare_create_file(sys_passwd) != 0) {
         return 0;
     }
 
-    output = fopen(file, "wt");
+    output = fopen(file, "wx");
 
     selinux_after_create_file();
 
@@ -1168,7 +1174,7 @@ int write_passwd(const struct _node* passwd, const char* file) {
     }
 
     if (fclose(output)!=0) {
-	fprintf(stderr, "Error closing passwd-file: %s\n", strerror(errno));
+	fprintf(stderr, "Error closing passwd-file %s: %s\n", file, strerror(errno));
 	return 0;
     }
 
@@ -1182,11 +1188,14 @@ int write_shadow(const struct _node* shadow, const char* file) {
     if (opt_verbose>2)
 	printf("Writing shadow-file to %s\n", file);
 
+    /* Delete possible left-over file */
+    (void) unlink(file);
+
     if (selinux_prepare_create_file(sys_shadow) != 0) {
         return 0;
     }
 
-    output = fopen(file, "wt");
+    output = fopen(file, "wx");
 
     selinux_after_create_file();
 
@@ -1205,7 +1214,7 @@ int write_shadow(const struct _node* shadow, const char* file) {
     }
 
     if (fclose(output)!=0) {
-	fprintf(stderr, "Error closing shadow-file: %s\n", strerror(errno));
+	fprintf(stderr, "Error closing shadow-file %s: %s\n", file, strerror(errno));
 	return 0;
     }
 
@@ -1232,11 +1241,15 @@ int write_group(const struct _node* group, const char* file) {
     if (opt_verbose>2)
 	printf("Writing group-file to %s\n", file);
 
+    /* Delete possible left-over file */
+    (void) unlink(file);
+
+
     if (selinux_prepare_create_file(sys_group) != 0) {
         return 0;
     }
 
-    output = fopen(file, "wt");
+    output = fopen(file, "wx");
 
     selinux_after_create_file();
 
@@ -1255,7 +1268,7 @@ int write_group(const struct _node* group, const char* file) {
     }
 
     if (fclose(output)!=0) {
-	fprintf(stderr, "Error closing group-file: %s\n", strerror(errno));
+	fprintf(stderr, "Error closing group-file %s: %s\n", file, strerror(errno));
 	return 0;
     }
 
@@ -1300,7 +1313,7 @@ int copy_filemodes(const char* source, const char* target) {
     }
 
     if (chmod(target, st.st_mode)!=0) {
-	fprintf(stderr, "Error chmoding %s: %s\n", source, strerror(errno));
+	fprintf(stderr, "Error chmoding %s: %s\n", target, strerror(errno));
 	return 0;
     }
 
@@ -1320,12 +1333,12 @@ int copy_filemodes(const char* source, const char* target) {
 	    if (!S_ISLNK (tst.st_mode) &&
 		    chown (target, st.st_uid, st.st_gid) != 0) {
 		fprintf(stderr, "Error lchowning %s: %s\n",
-			source, strerror(errno));
+			target, strerror(errno));
 		return 0;
 	    }
 	} else {
 	    fprintf(stderr, "Error lchowning %s: %s\n",
-		    source, strerror(errno));
+		    target, strerror(errno));
 	    return 0;
 	}
     }
@@ -1392,7 +1405,7 @@ int put_file_in_place(const char* source, const char* target) {
 
 /* Rewrite the account-database if we made any changes
  */
-int commit_files() {
+int commit_files(void) {
     char*	wf;
 
     if (!flag_dirty) {
@@ -1467,7 +1480,7 @@ int commit_files() {
 
 /* Try to lock the account database
  */
-int lock_files() {
+int lock_files(void) {
     if (lckpwdf()!=0) {
 	fprintf(stderr, "Error locking files: %s\n", strerror(errno));
 	return 0;
@@ -1479,7 +1492,7 @@ int lock_files() {
 
 /* Try to unlock the account database
  */
-int unlock_files() {
+int unlock_files(void) {
     if (ulckpwdf()!=0) {
 	fprintf(stderr, "Error unlocking files: %s\n", strerror(errno));
 	return 0;
